package test.drools.classloader;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author adixon
 */
public class FooClassLoader extends ClassLoader {

    public static final String FOO_CLASSNAME = "test.drools.classloader.Foo";
    private static final String FOO_CLASS_DATA = "Foo.class.test";
    
    private BuiltClasses _classes;

    public FooClassLoader(ClassLoader parent) {
        super(parent);
        _classes = new BuiltClasses(new String[] { FOO_CLASSNAME });
    }
    
    @Override
    protected synchronized Class<?> findClass(String name) throws ClassNotFoundException {
        final Class<?> found;
        if (_classes.contains(name)) {
            definePackageForClassIfNeeded(name);
            final byte[] byteCode = _classes.loadClassData(name);
            found = defineClass(name, byteCode, 0, byteCode.length);
        } else {
            found = super.findClass(name);
        }
        return found;
    }
    
    @Override
    public synchronized InputStream getResourceAsStream(String name) {
        final String asClassName = classFileNameToClassName(name);
        if (_classes.contains(asClassName)) {
            byte[] byteCode = _classes.loadClassData(asClassName);
            return new ByteArrayInputStream(byteCode);
        }
        return super.getResourceAsStream(name);
    }
    

    private synchronized void definePackageForClassIfNeeded(String name) {
        final String packageName = name.substring(0, name.lastIndexOf('.'));
        if (null == getPackage(packageName)) {
            definePackage(packageName, null, null, null, null, null, null, null);
        }
    }
    
    private static String classFileNameToClassName(final String classFileName) {
        return classFileName.replaceFirst("\\.class$", "").replace('/', '.');
    }
    
    /**
     * Javassist classes.
     * @author adixon
     */
    private final class BuiltClasses {
        
        private final List<String> _classNames;

        /** classFileName to byte[] */
        private final Map<String, byte[]> _cache = new HashMap<String, byte[]>();
        
        public BuiltClasses(String[] classNames) {
            _classNames = Arrays.asList(classNames);
        }
        
        boolean contains(String className) {
            return _classNames.contains(className);
        }
        
        byte[] loadClassData(String className) {
            if (_cache.containsKey(className)) {
                return _cache.get(className);
            }
            InputStream in = null;
            try {
                in = getClass().getResourceAsStream(FOO_CLASS_DATA);
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                byte[] b = new byte[1024];
                int len;
                while ((len = in.read(b)) > -1) {
                    baos.write(b, 0, len);
                }
                return baos.toByteArray();
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
            finally {
                try {
                    if (null != in) {
                        in.close();
                    }
                } catch (IOException e) {
                }
            }
        }
    }
    
}
